@@ -32,6 +32,7 @@
 #include <KStandardGuiItem>
 #include <KWindowSystem>
 
+#include <QClipboard>
 #include <QDesktopServices>
 #include <QJsonArray>
 #include <QPrintDialog>
@@ -59,8 +60,8 @@ KSMainWindow::KSMainWindow(bool onClickAvailable, QWidget *parent) :
     mClipboardButton(new QToolButton),
     mSaveButton(new QToolButton),
     mSaveMenu(new QMenu),
-    mSaveAsAction(new QAction(nullptr)),
-    mSaveAction(new QAction(nullptr)),
+    mSaveAsAction(new QAction(this)),
+    mSaveAction(new QAction(this)),
     mMessageWidget(new KMessageWidget),
     mToolsMenu(new QMenu),
     mScreenRecorderToolsMenu(new QMenu),
@@ -264,6 +265,7 @@ void KSMainWindow::moveEvent(QMoveEvent *event)
 void KSMainWindow::captureScreenshot(ImageGrabber::GrabMode mode, int timeout, bool includePointer, bool includeDecorations)
 {
     hide();
+    mMessageWidget->hide();
     emit newScreenshotRequest(mode, timeout, includePointer, includeDecorations);
 }
 
@@ -312,41 +314,71 @@ void KSMainWindow::openScreenshotsFolder()
     KIO::highlightInFileManager({location});
 }
 
-void KSMainWindow::showImageSharedFeedback(bool error, const QString &message)
+void KSMainWindow::quit(const QuitBehavior quitBehavior)
 {
-    if (error) {
-        mMessageWidget->setMessageType(KMessageWidget::Error);
-        mMessageWidget->setText(i18n("There was a problem sharing the image: %1", message));
+    qApp->setQuitOnLastWindowClosed(false);
+    hide();
+
+    if (quitBehavior == QuitBehavior::QuitImmediately) {
+        // Allow some time for clipboard content to transfer
+        // TODO: Find better solution
+        QTimer::singleShot(250, qApp, &QApplication::quit);
+    }
+    // TODO for else case:
+    // Currently it is expected that you emit forceNotify, and finally quit
+    // via a callback through KNotification::action1Activated. However, that
+    // is not working quite right, see Bug #389694 which needs fixing.
+}
+
+void KSMainWindow::showInlineMessage(const QString& message, const KMessageWidget::MessageType messageType, const MessageDuration messageDuration)
+{
+    mMessageWidget->setText(message);
+    mMessageWidget->setMessageType(messageType);
+
+    switch (messageType) {
+    case KMessageWidget::Error:
         mMessageWidget->setIcon(QIcon::fromTheme(QStringLiteral("dialog-error")));
-    } else {
-        mMessageWidget->setMessageType(KMessageWidget::Positive);
-        if (message.isEmpty())
-            mMessageWidget->setText(i18n("Image shared"));
-        else
-            mMessageWidget->setText(i18n("You can find the shared image at: <a href=\"%1\">%1</a>", message));
+        break;
+    case KMessageWidget::Positive:
         mMessageWidget->setIcon(QIcon::fromTheme(QStringLiteral("dialog-ok-apply")));
+        break;
+    case KMessageWidget::Information:
+        mMessageWidget->setIcon(QIcon::fromTheme(QStringLiteral("dialog-information")));
+        break;
+    default:
+        ;
     }
 
     mMessageWidget->animatedShow();
-    QTimer::singleShot(20000, mMessageWidget, &KMessageWidget::animatedHide);
+    if (messageDuration == MessageDuration::AutoHide) {
+        QTimer::singleShot(10000, mMessageWidget, &KMessageWidget::animatedHide);
+    }
+}
+
+void KSMainWindow::showImageSharedFeedback(bool error, const QString &message)
+{
+    if (error) {
+        showInlineMessage(i18n("There was a problem sharing the image: %1", message),
+                          KMessageWidget::Error);
+    } else {
+        if (message.isEmpty()) {
+            showInlineMessage(i18n("Image shared"), KMessageWidget::Positive);
+        } else {
+            showInlineMessage(i18n("The shared image link (<a href=\"%1\">%1</a>) has been copied to the clipboard.", message),
+                              KMessageWidget::Positive, MessageDuration::Persistent);
+            QApplication::clipboard()->setText(message);
+        }
+    }
 }
 
 void KSMainWindow::sendToClipboard()
 {
     ExportManager::instance()->doCopyToClipboard();
 
-    if (SpectacleConfig::instance()->quitAfterSaveOrCopyChecked()) {
-        qApp->setQuitOnLastWindowClosed(false);
-        hide();
-        QTimer::singleShot(250, qApp, &QApplication::quit);
-    }
-
-    mMessageWidget->setMessageType(KMessageWidget::Information);
-    mMessageWidget->setText(i18n("The screenshot has been copied to the clipboard."));
-    mMessageWidget->setIcon(QIcon::fromTheme(QStringLiteral("dialog-information")));
-
-    mMessageWidget->animatedShow();
-    QTimer::singleShot(10000, mMessageWidget, &KMessageWidget::animatedHide);
+    SpectacleConfig::instance()->quitAfterSaveOrCopyChecked()
+            ? quit()
+            : showInlineMessage(i18n("The screenshot has been copied to the clipboard."),
+                                KMessageWidget::Information);
 }
 
 void KSMainWindow::showPreferencesDialog()
@@ -366,13 +398,10 @@ void KSMainWindow::save()
     SpectacleConfig::instance()->setLastUsedSaveMode(SaveMode::Save);
     setDefaultSaveAction();
 
-    if (SpectacleConfig::instance()->quitAfterSaveOrCopyChecked()) {
-        ExportManager::instance()->doSave(QUrl(), true);
-        qApp->setQuitOnLastWindowClosed(false);
-        hide();
-    }
-    else {
-        ExportManager::instance()->doSave();
+    const bool quitChecked = SpectacleConfig::instance()->quitAfterSaveOrCopyChecked();
+    ExportManager::instance()->doSave(QUrl(), /* notify */ quitChecked);
+    if (quitChecked) {
+        quit(QuitBehavior::QuitExternally);
     }
 }
 
@@ -381,13 +410,8 @@ void KSMainWindow::saveAs()
     SpectacleConfig::instance()->setLastUsedSaveMode(SaveMode::SaveAs);
     setDefaultSaveAction();
 
-    if (SpectacleConfig::instance()->quitAfterSaveOrCopyChecked()) {
-        if (ExportManager::instance()->doSaveAs(this, true)) {
-            qApp->setQuitOnLastWindowClosed(false);
-            hide();
-        }
-    }
-    else {
-        ExportManager::instance()->doSaveAs(this, false);
+    const bool quitChecked = SpectacleConfig::instance()->quitAfterSaveOrCopyChecked();
+    if (ExportManager::instance()->doSaveAs(this, /* notify */ quitChecked) && quitChecked) {
+        quit(QuitBehavior::QuitExternally);
     }
 }
